@@ -4,7 +4,7 @@
 # Code used inside of a checker interpreter to ensure correct usage of
 # doctools formatting commands.
 #
-# Copyright (c) 2003-2010 Andreas Kupries <andreas_kupries@sourceforge.net>
+# Copyright (c) 2003-2024 Andreas Kupries <andreas_kupries@sourceforge.net>
 
 # L10N
 
@@ -252,6 +252,9 @@ proc vset {var args} {
 	0 {
 	    # Retrieve contents of variable VAR
 	    upvar #0 __$var data
+	    if {![info exists data]} {
+		return -code error "can't read doc variable \"$var\": not set"
+	    }
 	    return $data
 	}
 	1 {
@@ -271,6 +274,7 @@ proc vset {var args} {
 proc manpage_begin {title section version} {
     Enter manpage_begin
     if {[IsNot manpage_begin]} {Error mpbegin}
+    if {[string match {* *} $title]} {Error mptitle}
     Go header
     fmt_manpage_begin $title $section $version
 }
@@ -295,10 +299,10 @@ proc manpage_end {} {
     Go done
     fmt_manpage_end
 }
-proc require {pkg {version {}}} {
+proc require {pkg args} {
     Enter require
     if {[IsNot header]} {Error hdrcmd}
-    fmt_require $pkg $version
+    fmt_require $pkg $args
 }
 proc description {} {
     Enter description
@@ -308,7 +312,7 @@ proc description {} {
 }
 
 # Storage for (sub)section ids to enable checking for ambigous
-# identificaton. The ids on this level are logical names. The backends
+# identification. The ids on this level are logical names. The backends
 # are given physical names (via counters).
 global sect   ; # Map of logical -> physical ids
 global sectt  ; # Map of logical -> section title
@@ -612,7 +616,7 @@ proc sectref {id {title {}}} {
     }
 
     # If we have no text take the section title as text, if we
-    # can. Last fallback for thext is the id.
+    # can. Last fallback for text is the id.
     if {$title == {}} {
 	if {$pid != {}} {
 	    set title $sectt($fid)
