@@ -2,14 +2,12 @@
 #
 #	Implementation of doctoc objects for Tcl.
 #
-# Copyright (c) 2003-2009 Andreas Kupries <andreas_kupries@sourceforge.net>
+# Copyright (c) 2003-2019 Andreas Kupries <andreas_kupries@sourceforge.net>
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-# 
-# RCS: @(#) $Id: doctoc.tcl,v 1.21 2009/07/23 17:03:51 andreas_kupries Exp $
 
-package require Tcl 8.2
+package require Tcl 8.5 9
 package require textutil::expander
 
 # @mdgen OWNER: api_toc.tcl
@@ -184,7 +182,7 @@ proc ::doctools::toc::DocTocProc {name {cmd ""} args} {
     if { [llength [info level 0]] == 2 } {
 	error "wrong # args: should be \"$name option ?arg arg ...?\""
     }
-    
+
     # Split the args into command and args components
 
     if { [llength [info commands ::doctools::toc::_$cmd]] == 0 } {
@@ -277,7 +275,9 @@ proc ::doctools::toc::_configure {name args} {
 			upvar #0 ::doctools::toc::doctoc${name}::format format
 			set format $value
 		    } msg]} {
-			return -code error "doctools::toc::_configure: -format: $msg"
+			return -code error \
+			    -errorinfo $::errorInfo \
+			    "doctools::toc::_configure: -format: $msg"
 		    }
 		}
 		default {
@@ -511,14 +511,14 @@ proc ::doctools::toc::LookupFormat {name format} {
     # 2) Look for the file in the directories given to the object itself..
     # 3) Look for the file in the standard directories of this package.
 
-    if {[file exists $format]} {
+    if {[file exists $format] && [file isfile $format]} {
 	return $format
     }
 
     upvar #0 ::doctools::toc::doctoc${name}::paths opaths
     foreach path $opaths {
 	set f [file join $path toc.$format]
-	if {[file exists $f]} {
+	if {[file exists $f] && [file isfile $f]} {
 	    return $f
 	}
     }
@@ -526,7 +526,7 @@ proc ::doctools::toc::LookupFormat {name format} {
     variable paths
     foreach path $paths {
 	set f [file join $path toc.$format]
-	if {[file exists $f]} {
+	if {[file exists $f] && [file isfile $f]} {
 	    return $f
 	}
     }
@@ -563,6 +563,8 @@ proc ::doctools::toc::SetupFormatter {name format} {
     #$mpip eval [list source [file join $here api_toc.tcl]]
     interp alias $mpip dt_source   {} ::doctools::toc::Source  $mpip [file dirname $format]
     interp alias $mpip dt_read     {} ::doctools::toc::Read    $mpip [file dirname $format]
+    interp alias $mpip dt_package  {} ::doctools::toc::Package $mpip
+    interp alias $mpip file        {} ::doctools::toc::FileOp  $mpip
     interp alias $mpip puts_stderr {} ::puts stderr
     $mpip invokehidden source $format
     #$mpip eval [list source $format]
@@ -589,6 +591,7 @@ proc ::doctools::toc::SetupFormatter {name format} {
 	interp delete $mpip
 	error "$format error: Unable to query for number of passes"
     }
+    ##nagelfar ignore
     if {![string is integer $passes] || ($passes < 1)} {
 	interp delete $mpip
 	error "$format error: illegal number of passes \"$passes\""
@@ -631,7 +634,7 @@ proc ::doctools::toc::SetupFormatter {name format} {
 
     set format_ip  $mpip
     set formatfile $format
-    set xpasses    $passes
+    set xpasses    [format %d $passes]
     set xparam     $parameters
     return
 }
@@ -906,6 +909,47 @@ proc ::doctools::toc::Read {ip path file} {
     return [read [set f [open [file join $path [file tail $file]]]]][close $f]
 }
 
+proc ::doctools::toc::FileOp {ip args} {
+    #puts stderr "$ip (file $args)"
+    # -- FUTURE -- disallow unsafe operations --
+
+    return [eval [linsert $args 0 file]]
+}
+
+proc ::doctools::toc::Package {ip pkg} {
+    #puts stderr "$ip package require $pkg"
+
+    set indexScript [Locate $pkg]
+
+    $ip expose source
+    $ip expose load
+    $ip eval		$indexScript
+    $ip hide   source
+    $ip hide   load
+    #$ip eval [list source [file join $path [file tail $file]]]
+    return
+}
+
+proc ::doctools::toc::Locate {p} {
+    # @mdgen NODEP: doctools::__undefined__
+    catch {package require doctools::__undefined__}
+
+    #puts stderr "auto_path = [join $::auto_path \n]"
+
+    # Check if requested package is in the list of loadable packages.
+    # Then get the highest possible version, and then the index script
+
+    if {[lsearch -exact [package names] $p] < 0} {
+	return -code error "Unknown package $p"
+    }
+
+    set v  [lindex [lsort -increasing [package versions $p]] end]
+
+    #puts stderr "Package $p = $v"
+
+    return [package ifneeded $p $v]
+}
+
 #------------------------------------
 # Module initialization
 
@@ -920,4 +964,4 @@ namespace eval ::doctools::toc {
     catch {search [file join $here                             mpformats]}
 }
 
-package provide doctools::toc 1.1.2
+package provide doctools::toc 1.3.1
