@@ -3,8 +3,8 @@
 #   Tcl implementation of a general logging facility.
 #
 # Copyright (c) 2003      by David N. Welton <davidw@dedasys.com>
-# Copyright (c) 2004-2008 by Michael Schlenker <mic42@users.sourceforge.net>
-# Copyright (c) 2006      by Andreas Kupries <andreas_kupries@users.sourceforge.net>
+# Copyright (c) 2004-2011 by Michael Schlenker <mic42@users.sourceforge.net>
+# Copyright (c) 2006,2015 by Andreas Kupries <andreas_kupries@users.sourceforge.net>
 #
 # See the file license.terms.
 
@@ -13,8 +13,8 @@
 # This is accomplished through the use of Tcl namespaces.
 
 
-package require Tcl 8.2
-package provide logger 0.9
+package require Tcl 8.5 9
+package provide logger 0.9.5
 
 namespace eval ::logger {
     namespace eval tree {}
@@ -25,7 +25,7 @@ namespace eval ::logger {
 
     # The log 'levels'.
     variable levels [list debug info notice warn error critical alert emergency]
-    
+
     # The default global log level used for new logging services
     variable enabled "debug"
 
@@ -60,12 +60,12 @@ if {[package vcompare [package provide Tcl] 8.4] < 0} {
 # ::logger::_cmdPrefixExists --
 #
 # Utility function to check if a given callback prefix exists,
-# this should catch all oddities in prefix names, including spaces, 
+# this should catch all oddities in prefix names, including spaces,
 # glob patterns, non normalized namespaces etc.
 #
 # Arguments:
 #   prefix - The command prefix to check
-#   
+#
 # Results:
 #   1 or 0 for yes or no
 #
@@ -108,7 +108,12 @@ proc ::logger::init {service} {
     variable levels
     variable services
     variable enabled
-        
+
+    if {[string length [string trim $service {:}]] == 0} {
+        return -code error \
+               -errorcode [list LOGGER EMPTY_SERVICENAME] \
+               [::logger::mc "Service name invalid. May not consist only of : or be empty"]
+    }
     # We create a 'tree' namespace to house all the services, so
     # they are in a 'safe' namespace sandbox, and won't overwrite
     # any commands.
@@ -125,555 +130,555 @@ proc ::logger::init {service} {
     set [namespace current]::tree::${service}::levels $levels
     set [namespace current]::tree::${service}::oldname $service
     set [namespace current]::tree::${service}::enabled $enabled
-    
+
     namespace eval tree::${service} {
-    # Callback to use when the service in question is shut down.
-    variable delcallback [namespace current]::no-op
+	# Callback to use when the service in question is shut down.
+	variable delcallback [namespace current]::no-op
 
-    # Callback when the loglevel is changed
-    variable levelchangecallback [namespace current]::no-op
-    
-    # State variable to decide when to call levelcallback
-    variable inSetLevel 0
-    
-    # The currently configured levelcommands
-    variable lvlcmds 
-    array set lvlcmds {}
+	# Callback when the loglevel is changed
+	variable levelchangecallback [namespace current]::no-op
 
-    # List of procedures registered via the trace command
-    variable traceList ""
+	# State variable to decide when to call levelcallback
+	variable inSetLevel 0
 
-    # Flag indicating whether or not tracing is currently enabled
-    variable tracingEnabled 0
+	# The currently configured levelcommands
+	variable lvlcmds
+	array set lvlcmds {}
 
-    # We use this to disable a service completely.  In Tcl 8.4
-    # or greater, by using this, disabled log calls are a
-    # no-op!
+	# List of procedures registered via the trace command
+	variable traceList ""
 
-    proc no-op args {}
+	# Flag indicating whether or not tracing is currently enabled
+	variable tracingEnabled 0
+
+	# We use this to disable a service completely.  In Tcl 8.4
+	# or greater, by using this, disabled log calls are a
+	# no-op!
+
+	proc no-op args {}
+
+	proc stdoutcmd {level text} {
+	    variable service
+	    puts "\[[clock format [clock seconds]]\] \[$service\] \[$level\] \'$text\'"
+	}
+
+	proc stderrcmd {level text} {
+	    variable service
+	    puts stderr "\[[clock format [clock seconds]]\] \[$service\] \[$level\] \'$text\'"
+	}
 
 
-    proc stdoutcmd {level text} {
-        variable service
-        puts "\[[clock format [clock seconds]]\] \[$service\] \[$level\] \'$text\'"
-    }
-
-    proc stderrcmd {level text} {
-        variable service
-        puts stderr "\[[clock format [clock seconds]]\] \[$service\] \[$level\] \'$text\'"
-    }
+	# setlevel --
+	#
+	#   This command differs from enable and disable in that
+	#   it disables all the levels below that selected, and
+	#   then enables all levels above it, which enable/disable
+	#   do not do.
+	#
+	# Arguments:
+	#   lv - the level, as defined in $levels.
+	#
+	# Side Effects:
+	#   Runs disable for the level, and then enable, in order
+	#   to ensure that all levels are set correctly.
+	#
+	# Results:
+	#   None.
 
 
-    # setlevel --
-    #
-    #   This command differs from enable and disable in that
-    #   it disables all the levels below that selected, and
-    #   then enables all levels above it, which enable/disable
-    #   do not do.
-    #
-    # Arguments:
-    #   lv - the level, as defined in $levels.
-    #
-    # Side Effects:
-    #   Runs disable for the level, and then enable, in order
-    #   to ensure that all levels are set correctly.
-    #
-    # Results:
-    #   None.
+	proc setlevel {lv} {
+	    variable inSetLevel 1
+	    set oldlvl [currentloglevel]
 
+	    # do not allow enable and disable to do recursion
+	    if {[catch {
+		disable $lv 0
+		set newlvl [enable $lv 0]
+	    } msg] == 1} {
+		return -code error -errorcode $::errorCode $msg
+	    }
+	    # do the recursion here
+	    logger::walk [namespace current] [list setlevel $lv]
 
-    proc setlevel {lv} {
-        variable inSetLevel 1
-        set oldlvl [currentloglevel]
-        
-        # do not allow enable and disable to do recursion
-        if {[catch {
-            disable $lv 0
-            set newlvl [enable $lv 0]
-        } msg] == 1} {
-            return -code error -errorcode $::errorCode $msg
-        }
-        # do the recursion here
-        logger::walk [namespace current] [list setlevel $lv]
-        
-        set inSetLevel 0
-        lvlchangewrapper $oldlvl $newlvl
-        return
-    }
+	    set inSetLevel 0
+	    lvlchangewrapper $oldlvl $newlvl
+	    return
+	}
 
-    # enable --
-    #
-    #   Enable a particular 'level', and above, for the
-    #   service, and its 'children'.
-    #
-    # Arguments:
-    #   lv - the level, as defined in $levels.
-    #
-    # Side Effects:
-    #   Enables logging for the particular level, and all
-    #   above it (those more important).  It also walks
-    #   through all services that are 'children' and enables
-    #   them at the same level or above.
-    #
-    # Results:
-    #   None.
+	# enable --
+	#
+	#   Enable a particular 'level', and above, for the
+	#   service, and its 'children'.
+	#
+	# Arguments:
+	#   lv - the level, as defined in $levels.
+	#
+	# Side Effects:
+	#   Enables logging for the particular level, and all
+	#   above it (those more important).  It also walks
+	#   through all services that are 'children' and enables
+	#   them at the same level or above.
+	#
+	# Results:
+	#   None.
 
-    proc enable {lv {recursion 1}} {
-        variable levels
-        set lvnum [lsearch -exact $levels $lv]
-        if { $lvnum == -1 } {
-        return -code error \
-               -errorcode [list LOGGER INVALID_LEVEL] \
-               [::logger::mc "Invalid level '%s' - levels are %s" $lv $levels]
-        }
+	proc enable {lv {recursion 1}} {
+	    variable levels
+	    set lvnum [lsearch -exact $levels $lv]
+	    if { $lvnum == -1 } {
+		return -code error \
+		    -errorcode [list LOGGER INVALID_LEVEL] \
+		    [::logger::mc "Invalid level '%s' - levels are %s" $lv $levels]
+	    }
 
-        variable enabled
-        set newlevel $enabled
-        set elnum [lsearch -exact $levels $enabled]
-        if {($elnum == -1) || ($elnum > $lvnum)} {
-            set newlevel $lv
-        }
-                
-        variable service
-        while { $lvnum <  [llength $levels] } {
-        interp alias {} [namespace current]::[lindex $levels $lvnum] \
-            {} [namespace current]::[lindex $levels $lvnum]cmd
-        incr lvnum
-        }
-        
-        if {$recursion} {
-            logger::walk [namespace current] [list enable $lv]
-        }
-        lvlchangewrapper $enabled $newlevel
-        set enabled $newlevel
-    }
+	    variable enabled
+	    set newlevel $enabled
+	    set elnum [lsearch -exact $levels $enabled]
+	    if {($elnum == -1) || ($elnum > $lvnum)} {
+		set newlevel $lv
+	    }
 
-    # disable --
-    #
-    #   Disable a particular 'level', and below, for the
-    #   service, and its 'children'.
-    #
-    # Arguments:
-    #   lv - the level, as defined in $levels.
-    #
-    # Side Effects:
-    #   Disables logging for the particular level, and all
-    #   below it (those less important).  It also walks
-    #   through all services that are 'children' and disables
-    #   them at the same level or below.
-    #
-    # Results:
-    #   None.
+	    variable service
+	    while { $lvnum <  [llength $levels] } {
+		interp alias {} [namespace current]::[lindex $levels $lvnum] \
+		    {} [namespace current]::[lindex $levels $lvnum]cmd
+		incr lvnum
+	    }
 
-    proc disable {lv {recursion 1}} {
-        variable levels
-        set lvnum [lsearch -exact $levels $lv]
-        if { $lvnum == -1 } {
-            return -code error \
-                   -errorcode [list LOGGER INVALID_LEVEL] \
-                   [::logger::mc "Invalid level '%s' - levels are %s" $lv $levels]
-        }
+	    if {$recursion} {
+		logger::walk [namespace current] [list enable $lv]
+	    }
+	    lvlchangewrapper $enabled $newlevel
+	    set enabled $newlevel
+	}
 
-        variable enabled
-        set newlevel $enabled
-        set elnum [lsearch -exact $levels $enabled]
-        if {($elnum > -1) && ($elnum <= $lvnum)} {
-            if {$lvnum+1 >= [llength $levels]} {
-                set newlevel "none"
-            } else {
-                set newlevel [lindex $levels [expr {$lvnum+1}]]
-            }
-        }
-        
-        while { $lvnum >= 0 } {
-        
-        interp alias {} [namespace current]::[lindex $levels $lvnum] {} \
-            [namespace current]::no-op
-        incr lvnum -1
-        }
-        if {$recursion} {
-            logger::walk [namespace current] [list disable $lv]
-        }
-        lvlchangewrapper $enabled $newlevel
-        set enabled $newlevel
-    }
+	# disable --
+	#
+	#   Disable a particular 'level', and below, for the
+	#   service, and its 'children'.
+	#
+	# Arguments:
+	#   lv - the level, as defined in $levels.
+	#
+	# Side Effects:
+	#   Disables logging for the particular level, and all
+	#   below it (those less important).  It also walks
+	#   through all services that are 'children' and disables
+	#   them at the same level or below.
+	#
+	# Results:
+	#   None.
 
-    # currentloglevel --
-    #
-    #   Get the currently enabled log level for this service.
-    #
-    # Arguments:
-    #   none
-    #
-    # Side Effects:
-    #   none
-    #
-    # Results:
-    #   current log level
-    #
+	proc disable {lv {recursion 1}} {
+	    variable levels
+	    set lvnum [lsearch -exact $levels $lv]
+	    if { $lvnum == -1 } {
+		return -code error \
+		    -errorcode [list LOGGER INVALID_LEVEL] \
+		    [::logger::mc "Invalid level '%s' - levels are %s" $lv $levels]
+	    }
 
-    proc currentloglevel {} {
-        variable enabled
-        return $enabled
-    }
+	    variable enabled
+	    set newlevel $enabled
+	    set elnum [lsearch -exact $levels $enabled]
+	    if {($elnum > -1) && ($elnum <= $lvnum)} {
+		if {$lvnum+1 >= [llength $levels]} {
+		    set newlevel "none"
+		} else {
+		    set newlevel [lindex $levels [expr {$lvnum+1}]]
+		}
+	    }
 
-    # lvlchangeproc --
-    #
-    #   Set or introspect a callback for when the logger instance 
-    #   changes its loglevel.
-    #
-    # Arguments:
-    #   cmd - the Tcl command to call, it is called with two parameters, old and new log level.
-    #   or none for introspection
-    #
-    # Side Effects:
-    #   None.
-    #
-    # Results:
-    #   If no arguments are given return the current callback cmd.
+	    while { $lvnum >= 0 } {
 
-    proc lvlchangeproc {args} {
-        variable levelchangecallback
-        
-        switch -exact -- [llength [::info level 0]] {
+		interp alias {} [namespace current]::[lindex $levels $lvnum] {} \
+		    [namespace current]::no-op
+		incr lvnum -1
+	    }
+	    if {$recursion} {
+		logger::walk [namespace current] [list disable $lv]
+	    }
+	    lvlchangewrapper $enabled $newlevel
+	    set enabled $newlevel
+	}
+
+	# currentloglevel --
+	#
+	#   Get the currently enabled log level for this service.
+	#
+	# Arguments:
+	#   none
+	#
+	# Side Effects:
+	#   none
+	#
+	# Results:
+	#   current log level
+	#
+
+	proc currentloglevel {} {
+	    variable enabled
+	    return $enabled
+	}
+
+	# lvlchangeproc --
+	#
+	#   Set or introspect a callback for when the logger instance
+	#   changes its loglevel.
+	#
+	# Arguments:
+	#   cmd - the Tcl command to call, it is called with two parameters, old and new log level.
+	#   or none for introspection
+	#
+	# Side Effects:
+	#   None.
+	#
+	# Results:
+	#   If no arguments are given return the current callback cmd.
+
+	proc lvlchangeproc {args} {
+	    variable levelchangecallback
+
+	    switch -exact -- [llength [::info level 0]] {
                 1   {return $levelchangecallback}
                 2   {
-                     if {[::logger::_cmdPrefixExists [lindex $args 0]]} {
+		    if {[::logger::_cmdPrefixExists [lindex $args 0]]} {
                         set levelchangecallback [lindex $args 0]
-                     } else {
+		    } else {
                         return -code error \
-                               -errorcode [list LOGGER INVALID_CMD] \
-                               [::logger::mc "Invalid cmd '%s' - does not exist" [lindex $args 0]]
-                     }    
-                    }
+			    -errorcode [list LOGGER INVALID_CMD] \
+			    [::logger::mc "Invalid cmd '%s' - does not exist" [lindex $args 0]]
+		    }
+		}
                 default {
                     return -code error \
-                           -errorcode [list LOGGER WRONG_NUM_ARGS] \
-                           [::logger::mc "Wrong # of arguments. Usage: \${log}::lvlchangeproc ?cmd?"]
+			-errorcode [list LOGGER WRONG_NUM_ARGS] \
+			[::logger::mc "Wrong # of arguments. Usage: \${log}::lvlchangeproc ?cmd?"]
                 }
-        }
-    }
+	    }
+	}
 
-    proc lvlchangewrapper {old new} {
-        variable inSetLevel
-        
-        # we are called after disable and enable are finished 
-        if {$inSetLevel} {return}
-        
-        # no action if level does not change
-        if {[string equal $old $new]} {return}
-        
-        variable levelchangecallback
-        # no action if levelchangecallback isn't a valid command
-        if {[::logger::_cmdPrefixExists $levelchangecallback]} {
-        catch {
-            uplevel \#0 [linsert $levelchangecallback end $old $new]
-        }
-        }
-    }
-    
-    # logproc --
-    #
-    #   Command used to create a procedure that is executed to
-    #   perform the logging.  This could write to disk, out to
-    #   the network, or something else.
-    #   If two arguments are given, use an existing command.
-    #   If three arguments are given, create a proc.
-    #
-    # Arguments:
-    #   lv - the level to log, which must be one of $levels.
-    #   args - either zero, one or two arguments.
-    #          if zero this returns the current command registered 
-    #          if one, this is a cmd name that is called for this level
-    #          if two, these are an argument and proc body
-    #
-    # Side Effects:
-    #   Creates a logging command to take care of the details
-    #   of logging an event.
-    #
-    # Results:
-    #   If called with zero length args, returns the name of the currently
-    #   configured logging procedure.
-    #   
-    #
+	proc lvlchangewrapper {old new} {
+	    variable inSetLevel
 
-    proc logproc {lv args} {
-        variable levels
-        variable lvlcmds
-        
-        set lvnum [lsearch -exact $levels $lv]
-        if { ($lvnum == -1) && ($lv != "trace") } {
-        return -code error \
-               -errorcode [list LOGGER INVALID_LEVEL] \
-               [::logger::mc "Invalid level '%s' - levels are %s" $lv $levels]
-        }
-        switch -exact -- [llength $args] {
-        0  {
-            return $lvlcmds($lv)
-           }
-        1  {
-            set cmd [lindex $args 0]
-            if {[string equal "[namespace current]::${lv}cmd" $cmd]} {return} 
-            if {[llength [::info commands $cmd]]} {
-                proc ${lv}cmd args [format {\
-                  uplevel 1 [list %s [expr {[llength $args]==1 ? [lindex $args end] : $args}]]
-                } $cmd]
-            } else {
-                return -code error \
-                       -errorcode [list LOGGER INVALID_CMD] \
-                       [::logger::mc "Invalid cmd '%s' - does not exist" $cmd]
-            }
-            set lvlcmds($lv) $cmd
-        }
-        2  {
-            foreach {arg body} $args {break}
-            proc ${lv}cmd args [format {\
-              _setservicename args
-              set val [%s [expr {[llength $args]==1 ? [lindex $args end] : $args}]]
-              _restoreservice
-              set val} ${lv}customcmd]
-            proc ${lv}customcmd $arg $body
-            set lvlcmds($lv) [namespace current]::${lv}customcmd
-        }
-        default {
-            return -code error \
-                   -errorcode [list LOGGER WRONG_USAGE] \
-                   [::logger::mc \
-                   "Usage: \${log}::logproc level ?cmd?\nor \${log}::logproc level argname body" ]
-        }
-        }
-    }
+	    # we are called after disable and enable are finished
+	    if {$inSetLevel} {return}
+
+	    # no action if level does not change
+	    if {[string equal $old $new]} {return}
+
+	    variable levelchangecallback
+	    # no action if levelchangecallback isn't a valid command
+	    if {[::logger::_cmdPrefixExists $levelchangecallback]} {
+		catch {
+		    uplevel \#0 [linsert $levelchangecallback end $old $new]
+		}
+	    }
+	}
+
+	# logproc --
+	#
+	#   Command used to create a procedure that is executed to
+	#   perform the logging.  This could write to disk, out to
+	#   the network, or something else.
+	#   If two arguments are given, use an existing command.
+	#   If three arguments are given, create a proc.
+	#
+	# Arguments:
+	#   lv - the level to log, which must be one of $levels.
+	#   args - either zero, one or two arguments.
+	#          if zero this returns the current command registered
+	#          if one, this is a cmd name that is called for this level
+	#          if two, these are an argument and proc body
+	#
+	# Side Effects:
+	#   Creates a logging command to take care of the details
+	#   of logging an event.
+	#
+	# Results:
+	#   If called with zero length args, returns the name of the currently
+	#   configured logging procedure.
+	#
+	#
+
+	proc logproc {lv args} {
+	    variable levels
+	    variable lvlcmds
+
+	    set lvnum [lsearch -exact $levels $lv]
+	    if { ($lvnum == -1) && ($lv != "trace") } {
+		return -code error \
+		    -errorcode [list LOGGER INVALID_LEVEL] \
+		    [::logger::mc "Invalid level '%s' - levels are %s" $lv $levels]
+	    }
+	    switch -exact -- [llength $args] {
+		0  {
+		    return $lvlcmds($lv)
+		}
+		1  {
+		    set cmd [lindex $args 0]
+		    if {[string equal "[namespace current]::${lv}cmd" $cmd]} {return}
+		    if {[llength [::info commands $cmd]]} {
+			proc ${lv}cmd args [format {
+			    uplevel 1 [list %s [expr {[llength $args]==1 ? [lindex $args end] : $args}]]
+			} $cmd]
+		    } else {
+			return -code error \
+			    -errorcode [list LOGGER INVALID_CMD] \
+			    [::logger::mc "Invalid cmd '%s' - does not exist" $cmd]
+		    }
+		    set lvlcmds($lv) $cmd
+		}
+		2  {
+		    foreach {arg body} $args {break}
+		    proc ${lv}cmd args [format {\
+						    _setservicename args
+			set val [%s [expr {[llength $args]==1 ? [lindex $args end] : $args}]]
+			_restoreservice
+			set val} ${lv}customcmd]
+		    proc ${lv}customcmd $arg $body
+		    set lvlcmds($lv) [namespace current]::${lv}customcmd
+		}
+		default {
+		    return -code error \
+			-errorcode [list LOGGER WRONG_USAGE] \
+			[::logger::mc \
+			     "Usage: \${log}::logproc level ?cmd?\nor \${log}::logproc level argname body" ]
+		}
+	    }
+	}
 
 
-    # delproc --
-    #
-    #   Set or introspect a callback for when the logger instance 
-    #   is deleted.
-    #
-    # Arguments:
-    #   cmd - the Tcl command to call.
-    #   or none for introspection
-    #
-    # Side Effects:
-    #   None.
-    #
-    # Results:
-    #   If no arguments are given return the current callback cmd.
+	# delproc --
+	#
+	#   Set or introspect a callback for when the logger instance
+	#   is deleted.
+	#
+	# Arguments:
+	#   cmd - the Tcl command to call.
+	#   or none for introspection
+	#
+	# Side Effects:
+	#   None.
+	#
+	# Results:
+	#   If no arguments are given return the current callback cmd.
 
-    proc delproc {args} {
-        variable delcallback
-        
-        switch -exact -- [llength [::info level 0]] {
+	proc delproc {args} {
+	    variable delcallback
+
+	    switch -exact -- [llength [::info level 0]] {
                 1   {return $delcallback}
                 2   { if {[::logger::_cmdPrefixExists [lindex $args 0]]} {
-                            set delcallback [lindex $args 0]
-                      } else {
-                        return -code error \
-                               -errorcode [list LOGGER INVALID_CMD] \
-                               [::logger::mc "Invalid cmd '%s' - does not exist" [lindex $args 0]]                      
-                      }
-                    }
+		    set delcallback [lindex $args 0]
+		} else {
+		    return -code error \
+			-errorcode [list LOGGER INVALID_CMD] \
+			[::logger::mc "Invalid cmd '%s' - does not exist" [lindex $args 0]]
+		}
+		}
                 default {
                     return -code error \
-                           -errorcode [list LOGGER WRONG_NUM_ARGS] \
-                           [::logger::mc "Wrong # of arguments. Usage: \${log}::delproc ?cmd?"]
+			-errorcode [list LOGGER WRONG_NUM_ARGS] \
+			[::logger::mc "Wrong # of arguments. Usage: \${log}::delproc ?cmd?"]
                 }
-        }
-    }
+	    }
+	}
 
 
-    # delete --
-    #
-    #   Delete the namespace and its children.
+	# delete --
+	#
+	#   Delete the namespace and its children.
 
-    proc delete {} {
-        variable delcallback
-        variable service
-        
-        logger::walk [namespace current] delete
-        if {[::logger::_cmdPrefixExists $delcallback]} {
-             uplevel \#0 [lrange $delcallback 0 end]  
-        } 
-        # clean up the global services list
-        set idx [lsearch -exact [logger::services] $service]
-        if {$idx !=-1} {
-            set ::logger::services [lreplace [logger::services] $idx $idx]
-        }
-        
-        namespace delete [namespace current]
-        
-    }
+	proc delete {} {
+	    variable delcallback
+	    variable service
 
-    # services --
-    #
-    #   Return all child services 
-    
-    proc services {} {
-        variable service
-        
-        set children [list]
-        foreach srv [logger::services] {
-            if {[string match "${service}::*" $srv]} {
-                lappend children $srv
-            }
-        }
-        return $children
-    }
+	    logger::walk [namespace current] delete
+	    if {[::logger::_cmdPrefixExists $delcallback]} {
+		uplevel \#0 [lrange $delcallback 0 end]
+	    }
+	    # clean up the global services list
+	    set idx [lsearch -exact [logger::services] $service]
+	    if {$idx !=-1} {
+		set ::logger::services [lreplace [logger::services] $idx $idx]
+	    }
 
-    # servicename --
-    #
-    #   Return the name of the service
-    
-    proc servicename {} {
-        variable service
-        return $service
-    }
-    
-    proc _setservicename {argname} {
-        variable service
-        variable oldname
-        upvar 1 $argname arg
-        if {[llength $arg] <= 1} {
-            return
-        }
-        
-        set count -1
-        set newname ""
-        while {[string equal [lindex $arg [expr {$count+1}]] "-_logger::service"]} {
-            incr count 2
-            set newname [lindex $arg $count]
-        }
-        if {[string equal $newname ""]} {
-            return
-        }
-        set oldname $service
-        set service $newname
-        # Pop off "-_logger::service <service>" from argument list
-        set arg [lreplace $arg 0 $count]
-    }
-        
-    proc _restoreservice {} {
-        variable service
-        variable oldname
-        set service $oldname
-        return
-    }
-    
-    proc trace { action args } {
-        variable service
+	    namespace delete [namespace current]
 
-        # Allow other boolean values (true, false, yes, no, 0, 1) to be used
-        # as synonymns for "on" and "off".
+	}
 
-        if {[string is boolean $action]} {
-            set xaction [expr {($action && 1) ? "on" : "off"}]
-        } else {
-            set xaction $action
-        }
+	# services --
+	#
+	#   Return all child services
 
-        # Check for required arguments for actions/subcommands and dispatch
-        # to the appropriate procedure.
+	proc services {} {
+	    variable service
 
-        switch -- $xaction {
-            "status" {
-                return [uplevel 1 [list logger::_trace_status $service $args]]
-            }
-            "on" {
-                if {[llength $args]} {
-                    return -code error \
-                           -errorcode [list LOGGER WRONG_NUM_ARGS] \
+	    set children [list]
+	    foreach srv [logger::services] {
+		if {[string match "${service}::*" $srv]} {
+		    lappend children $srv
+		}
+	    }
+	    return $children
+	}
+
+	# servicename --
+	#
+	#   Return the name of the service
+
+	proc servicename {} {
+	    variable service
+	    return $service
+	}
+
+	proc _setservicename {argname} {
+	    variable service
+	    variable oldname
+	    upvar 1 $argname arg
+	    if {[llength $arg] <= 1} {
+		return
+	    }
+
+	    set count -1
+	    set newname ""
+	    while {[string equal [lindex $arg [expr {$count+1}]] "-_logger::service"]} {
+		incr count 2
+		set newname [lindex $arg $count]
+	    }
+	    if {[string equal $newname ""]} {
+		return
+	    }
+	    set oldname $service
+	    set service $newname
+	    # Pop off "-_logger::service <service>" from argument list
+	    set arg [lreplace $arg 0 $count]
+	}
+
+	proc _restoreservice {} {
+	    variable service
+	    variable oldname
+	    set service $oldname
+	    return
+	}
+
+	proc trace { action args } {
+	    variable service
+
+	    # Allow other boolean values (true, false, yes, no, 0, 1) to be used
+	    # as synonymns for "on" and "off".
+
+	    if {[string is boolean $action]} {
+		set xaction [expr {($action && 1) ? "on" : "off"}]
+	    } else {
+		set xaction $action
+	    }
+
+	    # Check for required arguments for actions/subcommands and dispatch
+	    # to the appropriate procedure.
+
+	    switch -- $xaction {
+		"status" {
+		    return [uplevel 1 [list logger::_trace_status $service $args]]
+		}
+		"on" {
+		    if {[llength $args]} {
+			return -code error \
+			    -errorcode [list LOGGER WRONG_NUM_ARGS] \
                             [::logger::mc "wrong # args: should be \"trace on\""]
-                }
-                return [logger::_trace_on $service]
-            }
-            "off" {
-                if {[llength $args]} {
-                    return -code error \
-                           -errorcode [list LOGGER WRONG_NUM_ARGS] \
+		    }
+		    return [logger::_trace_on $service]
+		}
+		"off" {
+		    if {[llength $args]} {
+			return -code error \
+			    -errorcode [list LOGGER WRONG_NUM_ARGS] \
                             [::logger::mc "wrong # args: should be \"trace off\""]
-                }
-                return [logger::_trace_off $service]
-            }
-            "add" {
-                if {![llength $args]} {
-                    return -code error \
-                           -errorcode [list LOGGER WRONG_NUM_ARGS] \
-                           [::logger::mc "wrong # args: should be \"trace add ?-ns? <proc> ...\""]
-                }
-                return [uplevel 1 [list ::logger::_trace_add $service $args]]
-            }
-            "remove" {
-                if {![llength $args]} {
-                    return -code error \
-                           -errorcode [list LOGGER WRONG_NUM_ARGS] \
+		    }
+		    return [logger::_trace_off $service]
+		}
+		"add" {
+		    if {![llength $args]} {
+			return -code error \
+			    -errorcode [list LOGGER WRONG_NUM_ARGS] \
+			    [::logger::mc "wrong # args: should be \"trace add ?-ns? <proc> ...\""]
+		    }
+		    return [uplevel 1 [list ::logger::_trace_add $service $args]]
+		}
+		"remove" {
+		    if {![llength $args]} {
+			return -code error \
+			    -errorcode [list LOGGER WRONG_NUM_ARGS] \
                             [::logger::mc "wrong # args: should be \"trace remove ?-ns? <proc> ...\""]
-                }
-                return [uplevel 1 [list ::logger::_trace_remove $service $args]]
-            }
+		    }
+		    return [uplevel 1 [list ::logger::_trace_remove $service $args]]
+		}
 
-            default {
-	        return -code error \
-                       -errorcode [list LOGGER INVALID_ARG] \
-                    [::logger::mc "Invalid action \"%s\": must be status, add, remove,\
+		default {
+		    return -code error \
+			-errorcode [list LOGGER INVALID_ARG] \
+			[::logger::mc "Invalid action \"%s\": must be status, add, remove,\
                     on, or off" $action]
-            }
-        }
-    }
+		}
+	    }
+	}
 
-    # Walk the parent service namespaces to see first, if they
-    # exist, and if any are enabled, and then, as a
-    # consequence, enable this one
-    # too.
+	# Walk the parent service namespaces to see first, if they
+	# exist, and if any are enabled, and then, as a
+	# consequence, enable this one
+	# too.
 
-    enable $enabled
-    variable parent [namespace parent]
-    while {[string compare $parent "::logger::tree"]} {
-        # If the 'enabled' variable doesn't exist, create the
-        # whole thing.
-        if { ! [::info exists ${parent}::enabled] } {
-        
-        logger::init [string range $parent 16 end]
-        }
-        set enabled [set ${parent}::enabled]
-        enable $enabled
-        set parent [namespace parent $parent]
-    }
+	enable $enabled
+	variable parent [namespace parent]
+	while {[string compare $parent "::logger::tree"]} {
+	    # If the 'enabled' variable doesn't exist, create the
+	    # whole thing.
+	    if { ! [::info exists ${parent}::enabled] } {
+		logger::init [string range $parent 16 end]
+	    }
+	    set enabled [set ${parent}::enabled]
+	    enable $enabled
+	    set parent [namespace parent $parent]
+	}
     }
 
     # Now create the commands for different levels.
 
     namespace eval tree::${service} {
-    set parent [namespace parent]
+	set parent [namespace parent]
 
-    # We 'inherit' the commands from the parents.  This
-    # means that, if you want to share the same methods with
-    # children, they should be instantiated after the parent's
-    # methods have been defined.
-    if {[string compare $parent "::logger::tree"]} {
-        foreach lvl [::logger::levels] {
-            # OPTIMIZE: do not allow multiple aliases in the hierarchy
-            #           they can always be replaced by more efficient
-            #           direct aliases to the target procs.
-            interp alias {} [namespace current]::${lvl}cmd \
-                         {} ${parent}::${lvl}cmd -_logger::service $service
-        }
-        # inherit the starting loglevel of the parent service
-        setlevel [${parent}::currentloglevel]
+	# We 'inherit' the commands from the parents.  This
+	# means that, if you want to share the same methods with
+	# children, they should be instantiated after the parent's
+	# methods have been defined.
 
-    } else {
-        foreach lvl [concat [::logger::levels] "trace"] {
-            proc ${lvl}cmd args [format {\
-              _setservicename args
-              set val [stdoutcmd %s [expr {[llength $args]==1 ? [lindex $args end] : $args}]]
-              _restoreservice
-              set val } $lvl]
+	variable lvl ; # prevent creative writing to the global scope
+	if {[string compare $parent "::logger::tree"]} {
+	    foreach lvl [::logger::levels] {
+		# OPTIMIZE: do not allow multiple aliases in the hierarchy
+		#           they can always be replaced by more efficient
+		#           direct aliases to the target procs.
+		interp alias {} [namespace current]::${lvl}cmd \
+		    {} ${parent}::${lvl}cmd -_logger::service $service
+	    }
+	    # inherit the starting loglevel of the parent service
+	    setlevel [${parent}::currentloglevel]
+	} else {
+	    foreach lvl [concat [::logger::levels] "trace"] {
+		proc ${lvl}cmd args [format {\
+						 _setservicename args
+		    set val [stdoutcmd %s [expr {[llength $args]==1 ? [lindex $args end] : $args}]]
+		    _restoreservice
+		    set val } $lvl]
 
-            set lvlcmds($lvl) [namespace current]::${lvl}cmd
-        }
+		set lvlcmds($lvl) [namespace current]::${lvl}cmd
+	    }
+	    setlevel $::logger::enabled
+	}
+	unset lvl ; # drop the temp iteration variable
     }
-    }
-    
-    
+
     return ::logger::tree::${service}
 }
 
@@ -741,8 +746,8 @@ proc ::logger::setlevel {lv} {
         return -code error \
                -errorcode [list LOGGER INVALID_LEVEL] \
                [::logger::mc "Invalid level '%s' - levels are %s" $lv $levels]
-    } 
-    set enabled $lv    
+    }
+    set enabled $lv
     if {[catch {
         foreach sv $services {
         ::logger::tree::${sv}::setlevel $lv
@@ -810,7 +815,7 @@ proc ::logger::servicecmd {service} {
 
 proc ::logger::import {args} {
     variable services
-    
+
     if {[llength $args] == 0 || [llength $args] > 7} {
     return -code error \
            -errorcode [list LOGGER WRONG_NUM_ARGS] \
@@ -819,7 +824,7 @@ proc ::logger::import {args} {
                         ?-force?\
                         ?-prefix prefix? ?-namespace namespace? service\""]
     }
-    
+
     # process options
     #
     set import_all 0
@@ -832,7 +837,7 @@ proc ::logger::import {args} {
         switch  -exact -- $opt {
             -all    { set import_all 1}
             -prefix { set prefix [lindex $args 0]
-                      set args [lrange $args 1 end]        
+                      set args [lrange $args 1 end]
                     }
             -namespace {
                       set ns [lindex $args 0]
@@ -851,22 +856,22 @@ proc ::logger::import {args} {
             }
         }
     }
-    
+
     #
     # build the list of commands to import
     #
-    
+
     set cmds [logger::levels]
     lappend cmds "trace"
     if {$import_all} {
-        lappend cmds setlevel enable disable logproc delproc services 
+        lappend cmds setlevel enable disable logproc delproc services
         lappend cmds servicename currentloglevel delete
     }
-    
+
     #
     # check the service argument
     #
-    
+
     set service [lindex $args 0]
     if {[lsearch -exact $services $service] == -1} {
             return -code error \
@@ -878,25 +883,25 @@ proc ::logger::import {args} {
     # setup the namespace for the import
     #
 
-    set sourcens [logger::servicecmd $service]     
+    set sourcens [logger::servicecmd $service]
     set localns  [uplevel 1 namespace current]
-    
+
     if {[string match ::* $ns]} {
         set importns $ns
     } else {
         set importns ${localns}::$ns
-    }    
+    }
 
     # fake namespace exists for Tcl 8.2 - 8.3
     if {![_nsExists $importns]} {
         namespace eval $importns {}
-    } 
+    }
+
 
-    
     #
     # prepare the import
     #
-    
+
     set imports ""
     foreach cmd $cmds {
         set cmdname ${importns}::${prefix}$cmd
@@ -908,11 +913,11 @@ proc ::logger::import {args} {
         }
         lappend imports ${importns}::${prefix}$cmd ${sourcens}::${cmd}
     }
-    
+
     #
     # and execute the aliasing after checking all is well
     #
-    
+
     foreach {target source} $imports {
         proc $target {args} "uplevel 1 \[linsert \$args 0 $source \]"
     }
@@ -934,11 +939,25 @@ proc ::logger::import {args} {
 # Results:
 #   none
 
-proc ::logger::initNamespace {ns {level warn}} {
+proc ::logger::initNamespace {ns {level {}}} {
     set service [string trimleft $ns :]
+    if {$level == ""} {
+	# No user-specified level. Figure something out.
+	# - If the parent service exists then the 'logger::init'
+	#   below will automatically inherit its level. Good enough.
+	# - Without a parent service go and use a default level of 'warn'.
+	set parent    [string trimleft [namespace qualifiers $service] :]
+	set hasparent [expr {($parent != {}) && [_nsExists ::logger::tree::${parent}]}]
+	if {!$hasparent} {
+	    set level warn
+	}
+    }
+
     namespace eval $ns [list ::logger::init $service]
     namespace eval $ns [list ::logger::import -force -all -namespace log $service]
-    namespace eval $ns [list log::setlevel $level]
+    if {$level != ""} {
+	namespace eval $ns [list log::setlevel $level]
+    }
     return
 }
 
