@@ -15,15 +15,11 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
-#
-# $Id: md5x.tcl,v 1.19 2008/07/04 18:27:00 andreas_kupries Exp $
 
-package require Tcl 8.2;                # tcl minimum version
+package require Tcl 8.5 9;                # tcl minimum version
 
 namespace eval ::md5 {
-    variable version 2.0.7
-    variable rcsid {$Id: md5x.tcl,v 1.19 2008/07/04 18:27:00 andreas_kupries Exp $}
-    variable accel
+    variable  accel
     array set accel {critcl 0 cryptkit 0 trf 0}
 
     namespace export md5 hmac MD5Init MD5Update MD5Final
@@ -108,7 +104,7 @@ proc ::md5::MD5Update {token data} {
     }
 
     # Update the state values
-    incr state(n) [string length $data]
+    incr   state(n) [string length $data]
     append state(i) $data
 
     # Calculate the hash for any complete blocks
@@ -165,20 +161,29 @@ proc ::md5::MD5Final {token} {
     if {$pad == 0} {
         incr pad 64
     }
-    append state(i) [binary format a$pad \x80]
 
+    # puts "P $pad|bits=[expr {8 * $state(n)}]"
+    append state(i) [binary format a$pad \x80]
+    
     # RFC1321:3.2 - Append length in bits as little-endian wide int.
     append state(i) [binary format ii [expr {8 * $state(n)}] 0]
 
+    #puts DATA=[Hex $state(i)]([string length $state(i)])
+
     # Calculate the hash for the remaining block.
     set len [string length $state(i)]
     for {set n 0} {($n + 64) <= $len} {} {
         MD5Hash $token [string range $state(i) $n [incr n 64]]
     }
 
+    #puts md5-post__________________________________________
+    #parray ::${token}
+
     # RFC1321:3.5 - Output
     set r [bytes $state(A)][bytes $state(B)][bytes $state(C)][bytes $state(D)]
     unset state
+
+    #puts HASH=[Hex $r]
     return $r
 }
 
@@ -262,9 +267,13 @@ set ::md5::MD5Hash_body {
     variable $token
     upvar 0 $token state
 
+    # puts TR__=[Hex $msg]([string length $msg])
+
     # RFC1321:3.4 - Process Message in 16-Word Blocks
     binary scan $msg i* blocks
     foreach {X0 X1 X2 X3 X4 X5 X6 X7 X8 X9 X10 X11 X12 X13 X14 X15} $blocks {
+        #puts BL
+
         set A $state(A)
         set B $state(B)
         set C $state(C)
@@ -458,6 +467,9 @@ regsub -all -line \
 
 # RFC 1321:3.4 step 4: inline the set of constant modifiers.
 namespace eval md5 {
+    variable tName
+    variable tVal
+    variable map
     foreach tName {
         T01 T02 T03 T04 T05 T06 T07 T08 T09 T10 
         T11 T12 T13 T14 T15 T16 T17 T18 T19 T20 
@@ -525,7 +537,7 @@ proc ::md5::LoadAccelerator {name} {
         critcl {
             if {![catch {package require tcllibc}]
                 || ![catch {package require md5c}]} {
-                set r [expr {[info command ::md5::md5c] != {}}]
+                set r [expr {[info commands ::md5::md5c] != {}}]
             }
         }
         cryptkit {
@@ -606,7 +618,16 @@ proc ::md5::md5 {args} {
                 should be \"md5 ?-hex? -filename file | string\""
         }
         set tok [MD5Init]
+
+        #puts md5_______________________________________________
+        #parray ::${tok}
+
+        #puts IN=(([lindex $args 0]))
         MD5Update $tok [lindex $args 0]
+
+        #puts md5-final_________________________________________
+        #parray ::${tok}
+
         set r [MD5Final $tok]
 
     } else {
@@ -706,7 +727,7 @@ namespace eval ::md5 {
     unset    e
 }
 
-package provide md5 $::md5::version
+package provide md5 2.0.9
 
 # -------------------------------------------------------------------------
 # Local Variables:
