@@ -22,16 +22,12 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
-#
-# $Id: ripemd128.tcl,v 1.10 2009/05/07 01:12:59 patthoyts Exp $
 
-package require Tcl 8.2;                # tcl minimum version
+package require Tcl 8.5 9;                # tcl minimum version
 
 namespace eval ::ripemd {
     namespace eval ripemd128 {
-        variable version 1.0.4
-        variable rcsid {$Id: ripemd128.tcl,v 1.10 2009/05/07 01:12:59 patthoyts Exp $}
-        variable accel
+        variable  accel
         array set accel {trf 0}
 
         variable uid
@@ -530,7 +526,7 @@ proc ::ripemd::ripemd128::LoadAccelerator {name} {
         #critcl {
         #    if {![catch {package require tcllibc}]
         #        || ![catch {package require sha1c}]} {
-        #        set r [expr {[info command ::sha1::sha1c] != {}}]
+        #        set r [expr {[info commands ::sha1::sha1c] != {}}]
         #    }
         #}
         #cryptkit {
@@ -719,16 +715,17 @@ namespace eval ::ripemd {
 
 # Try and load a compiled extension to help.
 namespace eval ::ripemd::ripemd128 {
-    foreach e {trf} { if {[LoadAccelerator $e]} { break } }
+    variable e {}
+    foreach e {trf} {
+        if {[LoadAccelerator $e]} break
+    }
     unset e
 }
 
-package provide ripemd128 $::ripemd::ripemd128::version
+package provide ripemd128 1.0.6
 
 # -------------------------------------------------------------------------
 # Local Variables:
 #   mode: tcl
 #   indent-tabs-mode: nil
 # End:
-
-
