@@ -22,17 +22,13 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
-#
-# $Id: ripemd160.tcl,v 1.8 2009/05/07 01:12:59 patthoyts Exp $
 
-package require Tcl 8.2;                # tcl minimum version
+package require Tcl 8.5 9;                # tcl minimum version
 #catch {package require ripemdc 1.0};   # tcllib critcl alternative
 
 namespace eval ::ripemd {
     namespace eval ripemd160 {
-        variable version 1.0.4
-        variable rcsid {$Id: ripemd160.tcl,v 1.8 2009/05/07 01:12:59 patthoyts Exp $}
-        variable accel
+        variable  accel
         array set accel {cryptkit 0 trf 0}
 
         variable uid
@@ -539,8 +535,9 @@ proc ::ripemd::ripemd160::<<< {v n} {
 namespace eval ::ripemd::ripemd160 {
 
     # Inline function FF and FFF
-    set Split {(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\d+)}
+    variable Split {(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\d+)}
     
+    ##nagelfar ignore
     regsub -all -line \
         "^\\s+FFF?\\s+$Split$" \
         $RIPEMD160Hash_body \
@@ -549,6 +546,7 @@ namespace eval ::ripemd::ripemd160 {
         RIPEMD160Hash_body
     
     # Inline function GG
+    ##nagelfar ignore
     regsub -all -line \
         "^\\s+GG\\s+$Split$" \
         $RIPEMD160Hash_body \
@@ -558,6 +556,7 @@ namespace eval ::ripemd::ripemd160 {
         RIPEMD160Hash_body
 
     # Inline function GGG
+    ##nagelfar ignore
     regsub -all -line \
         "^\\s+GGG\\s+$Split$" \
         $RIPEMD160Hash_body \
@@ -567,6 +566,7 @@ namespace eval ::ripemd::ripemd160 {
         RIPEMD160Hash_body
 
     # Inline function HH
+    ##nagelfar ignore
     regsub -all -line \
         "^\\s+HH\\s+$Split$" \
         $RIPEMD160Hash_body \
@@ -576,6 +576,7 @@ namespace eval ::ripemd::ripemd160 {
         RIPEMD160Hash_body
 
     # Inline function HHH
+    ##nagelfar ignore
     regsub -all -line \
         "^\\s+HHH\\s+$Split$" \
         $RIPEMD160Hash_body \
@@ -585,6 +586,7 @@ namespace eval ::ripemd::ripemd160 {
         RIPEMD160Hash_body
 
     # Inline function II
+    ##nagelfar ignore
     regsub -all -line \
         "^\\s+II\\s+$Split$" \
         $RIPEMD160Hash_body \
@@ -594,6 +596,7 @@ namespace eval ::ripemd::ripemd160 {
         RIPEMD160Hash_body
 
     # Inline function III
+    ##nagelfar ignore
     regsub -all -line \
         "^\\s+III\\s+$Split$" \
         $RIPEMD160Hash_body \
@@ -603,6 +606,7 @@ namespace eval ::ripemd::ripemd160 {
         RIPEMD160Hash_body
 
     # Inline function JJ
+    ##nagelfar ignore
     regsub -all -line \
         "^\\s+JJ\\s+$Split$" \
         $RIPEMD160Hash_body \
@@ -612,6 +616,7 @@ namespace eval ::ripemd::ripemd160 {
         RIPEMD160Hash_body
 
     # Inline function JJJ
+    ##nagelfar ignore
     regsub -all -line \
         "^\\s+JJJ\\s+$Split$" \
         $RIPEMD160Hash_body \
@@ -621,6 +626,7 @@ namespace eval ::ripemd::ripemd160 {
         RIPEMD160Hash_body
 
     # Inline simple <<<
+    ##nagelfar ignore
     regsub -all -line \
         {\[<<< (\$\S+)\s+(\d+)\]$} \
         $RIPEMD160Hash_body \
@@ -663,7 +669,7 @@ proc ::ripemd::ripemd160::LoadAccelerator {name} {
         #critcl {
         #    if {![catch {package require tcllibc}]
         #        || ![catch {package require sha1c}]} {
-        #        set r [expr {[info command ::sha1::sha1c] != {}}]
+        #        set r [expr {[info commands ::sha1::sha1c] != {}}]
         #    }
         #}
         cryptkit {
@@ -852,16 +858,17 @@ namespace eval ::ripemd {
 
 # Try and load a compiled extension to help.
 namespace eval ::ripemd::ripemd160 {
-    foreach e {cryptkit trf} { if {[LoadAccelerator $e]} { break } }
+    variable e {}
+    foreach e {cryptkit trf} {
+        if {[LoadAccelerator $e]} break
+    }
     unset e
 }
 
-package provide ripemd160 $::ripemd::ripemd160::version
+package provide ripemd160 1.0.7
 
 # -------------------------------------------------------------------------
 # Local Variables:
 #   mode: tcl
 #   indent-tabs-mode: nil
 # End:
-
-
