@@ -15,12 +15,12 @@
 #
 #-----------------------------------------------------------------------
 
-package require Tcl 8.5
+package require Tcl 8.5 9
 
 # Define the snit namespace and save the library directory
 
 namespace eval ::snit:: {
-    set library [file dirname [info script]]
+    variable library [file dirname [info script]]
 }
 
 # Load the kernel.
@@ -29,4 +29,4 @@ source [file join $::snit::library main2.tcl]
 # Load the library of Snit validation types.
 source [file join $::snit::library validate.tcl]
 
-package provide snit 2.3.2
+package provide snit 2.3.4
