@@ -9,11 +9,9 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
-# RCS: @(#) $Id: list.tcl,v 1.27 2011/09/17 14:35:36 mic42 Exp $
-#
 #----------------------------------------------------------------------
 
-package require Tcl 8.4
+package require Tcl 8.5 9
 package require cmdline
 
 namespace eval ::struct { namespace eval list {} }
@@ -647,7 +645,10 @@ proc ::struct::list::Lflatten {args} {
     while {[string match -* [set opt [::lindex $args 0]]]} {
 	switch -glob -- $opt {
 	    -full   {set full 1}
-	    --      {break}
+	    --      {
+                set args [::lrange $args 1 end]
+                break ; # fix ticket 6e778502b8 -- break exits while loop
+            }
 	    default {
 		return -code error "Unknown option \"$opt\", should be either -full, or --"
 	    }
@@ -758,7 +759,7 @@ proc ::struct::list::Lmapfor {var sequence script} {
 proc ::struct::list::Lfilter {sequence cmdprefix} {
     # Shortcut when nothing is to be done.
     if {[::llength $sequence] == 0} {return $sequence}
-    return [Lfold $sequence {} [::list ::struct::list::FTest $cmdprefix]]
+    return [uplevel 1 [::list ::struct::list::Lfold $sequence {} [::list ::struct::list::FTest $cmdprefix]]]
 }
 
 proc ::struct::list::FTest {cmdprefix result item} {
@@ -822,7 +823,8 @@ proc ::struct::list::Lsplit {sequence cmdprefix args} {
     if {$largs == 0} {
 	# Shortcut when nothing is to be done.
 	if {[::llength $sequence] == 0} {return {{} {}}}
-	return [Lfold $sequence {} [::list ::struct::list::PFTest $cmdprefix]]
+	return [uplevel 1 [::list [namespace which Lfold] $sequence {} [
+		::list ::struct::list::PFTest $cmdprefix]]]
     } elseif {$largs == 2} {
 	# Shortcut when nothing is to be done.
 	foreach {pv fv} $args break
@@ -832,7 +834,9 @@ proc ::struct::list::Lsplit {sequence cmdprefix args} {
 	    set fail {}
 	    return {0 0}
 	}
-	foreach {pass fail} [Lfold $sequence {} [::list ::struct::list::PFTest $cmdprefix]] break
+	foreach {pass fail} [uplevel 1 [
+		::list ::struct::list::Lfold $sequence {} [
+			::list ::struct::list::PFTest $cmdprefix]]] break
 	return [::list [llength $pass] [llength $fail]]
     } else {
 	return -code error \
@@ -938,12 +942,39 @@ proc ::struct::list::Ldelete {var item} {
 # Side effects:
 #       None
 
-proc ::struct::list::Lequal {a b} {
+proc ::struct::list::Lequal {a b args} {
+    # Author of this command is "Richard Suchenwirth"
+
+    set args [::lreverse [::lassign [::lreverse [::list $a $b {*}$args]] a b]]
+    # a b args --> args a b
+    set recurse 1
+    while {[string match -* [set o [::lindex $args 0]]]} {
+	set args [::lrange $args 1 end]
+	switch -exact -- $o {
+	    --      break
+	    -simple { set recurse 0 }
+	    default {
+		return -code error "Bad option \"$o\", expected -simple, or --"
+	    }
+	}
+    }
+    if {[::llength $args]} {
+	return -code error "too many arguments after options"
+    }
+    return [EqualCore $recurse $a $b]
+}
+
+proc ::struct::list::EqualCore {recurse a b} {
     # Author of this command is "Richard Suchenwirth"
 
     if {[::llength $a] != [::llength $b]} {return 0}
     if {[::lindex $a 0] == $a && [::lindex $b 0] == $b} {return [string equal $a $b]}
-    foreach i $a j $b {if {![Lequal $i $j]} {return 0}}
+
+    if {$recurse} {
+	foreach i $a j $b {if {![EqualCore $recurse $i $j]} {return 0}}
+    } else {
+	foreach i $a j $b {if {$i ne $j} {return 0}}
+    }
     return 1
 }
 
@@ -996,6 +1027,7 @@ proc ::struct::list::Lrepeatn {value args} {
 if { [package vcompare [package provide Tcl] 8.5] < 0 } {
 
     proc ::struct::list::Lrepeat {positiveCount value args} {
+	##nagelfar ignore
 	if {![string is integer -strict $positiveCount]} {
 	    return -code error "expected integer but got \"$positiveCount\""
 	} elseif {$positiveCount < 1} {
@@ -1653,7 +1685,7 @@ proc ::struct::list::Lnextperm {perm} {
     # following aj. Let its index be l, and interchange aj and al.
 
     set l $len
-    while { $aj >= [set al [lindex $perm $l]] } {
+    while { [string compare $aj [set al [lindex $perm $l]]] >= 0 } {
 	incr l -1
     }
     lset perm $j $al
@@ -1825,4 +1857,4 @@ namespace eval ::struct {
     namespace import -force list::list
     namespace export list
 }
-package provide struct::list 1.8.1
+package provide struct::list 1.9
