@@ -1,7 +1,5 @@
-#! /bin/sh
-# the next line restarts with tclsh \
-exec tclsh8.5 "$0" ${1+"$@"}
-
+#! /usr/local/bin/tclsh8.6
+## -*- tcl -*-
 #####
 #
 # "BibTeX parser" -- Example Application.
@@ -20,7 +18,7 @@ exec tclsh8.5 "$0" ${1+"$@"}
 # Copyright (c) 2005 Neil Madden.
 # License: Tcl/BSD style.
 
-package require Tcl 8.4
+package require Tcl 8.5 9
 package require bibtex
 
 proc readfile file {
