@@ -1,7 +1,5 @@
-#! /bin/sh
-# -*- tcl -*- \
-exec tclsh8.5 "$0" ${1+"$@"}
-
+#! /usr/local/bin/tclsh8.6
+## -*- tcl -*-
 #   - simple tcl/tk test script for FTP library package -
 #
 #   Required:	tcl/tk8.3
@@ -22,7 +20,7 @@ exec tclsh8.5 "$0" ${1+"$@"}
 #
 ########################################################################
 
-package require Tcl 8.3
+package require Tcl 8.5 9
 package require Tk
 package require ftp 2.0
 
