@@ -1,7 +1,5 @@
-#! /bin/sh
-# -*- tcl -*- \
-exec tclsh8.5 "$0" ${1+"$@"}
-
+#! /usr/local/bin/tclsh8.6
+## -*- tcl -*-
 #  - homepage update program using FTP -
 #
 #   Required:   tcl/tk8.2
@@ -31,7 +29,7 @@ exec tclsh8.5 "$0" ${1+"$@"}
 ########################################################################
 
 # load required FTP package library 
-package require Tcl 8.3
+package require Tcl 8.5 9
 package require ftp 2.0
 package require Tk
 if {![llength [info commands tkButtonInvoke]]} {
@@ -74,9 +72,9 @@ menu .menu.file -tearoff 0
 
 #menu .menu.edit -tearoff 0
 #.menu add cascade -label "Bearbeiten" -menu .menu.edit -underline 0
-#.menu.edit add command -label "Alle Lschen" -underline 0 -state disabled -command {
+#.menu.edit add command -label "Alle Löschen" -underline 0 -state disabled -command {
 #	.view.remote.list selection set 0 end; BusyCommand DeleteremoteFiles}
-#.menu.edit add command -label "Alle bertragen" -underline 0 -state disabled -command Quit
+#.menu.edit add command -label "Alle Übertragen" -underline 0 -state disabled -command Quit
 
 menu .menu.view -tearoff 0
 .menu add cascade -label "View" -menu .menu.view -underline 0
@@ -798,17 +796,8 @@ global ftp
 }
 
 
-if {[package vcompare [info tclversion] 8.4] >= 0} {
-    proc Touch {filename} {
-	file mtime $filename [clock seconds]
-    }
-} else {
-    # update timestamp
-    proc Touch {filename} {
-	set file [open $filename w]
-	puts -nonewline $file ""
-	close $file
-    }
+proc Touch {filename} {
+    file mtime $filename [clock seconds]
 }
 
 
