@@ -1,8 +1,7 @@
-#! /bin/sh
-# -*- tcl -*- \
-exec tclsh8.5 "$0" ${1+"$@"}
+#! /usr/local/bin/tclsh8.6
+## -*- tcl -*-
 
-package require Tcl 8.3
+package require Tcl 8.5 9
 package require ftp 2.0
 
 # user configuration
