@@ -1,6 +1,5 @@
-#! /bin/sh
-# -*- tcl -*- \
-exec tclsh8.5 "$0" ${1+"$@"}
+#! /usr/local/bin/tclsh8.6
+## -*- tcl -*-
 
 # irc example script, by David N. Welton <davidw@dedasys.com>
 # $Id: irc_example.tcl,v 1.10 2009/01/30 04:18:14 andreas_kupries Exp $
