@@ -1,7 +1,5 @@
-#! /bin/sh
-# -*- tcl -*- \
-exec tclsh8.5 "$0" ${1+"$@"}
-
+#! /usr/local/bin/tclsh8.6
+## -*- tcl -*-
 # personal.tcl - process personal mail
 #
 # (c) 1999 Marshall T. Rose
@@ -10,7 +8,8 @@ exec tclsh8.5 "$0" ${1+"$@"}
 # The original version was written in 1994!
 #
 
-package require Tcl 8.3
+package require Tcl 8.5 9
+package require file::home	;# tcllib file home forward compatibility
 
 global options
 
@@ -445,7 +444,7 @@ if {[catch {
 
         lappend deleteFiles [set messageFile $tmp(file)]
 
-        catch { file attributes $messageFile -permissions 0600 }
+        catch { file attributes $messageFile -permissions 0o600 }
 
         if {[gets stdin line] <= 0} {
             cleanup "empty message"
@@ -468,13 +467,14 @@ if {[catch {
         if {[catch { id convert user $userName }]} {
             cleanup "userName doesn't exist: $userName"
         }
-        if {([catch { file isdirectory ~$userName } result]) \
+	set thisDir [file home $userName]
+        if {([catch { file isdirectory $thisDir } result]) \
                 || (!$result)} {
             cleanup "userName doesn't have a home directory: $userName"
         }
 
         umask 0077
-        cd ~$userName
+        cd $thisDir
     }
 
     if {![file exists $configFile]} {
