@@ -1,7 +1,5 @@
-#! /bin/sh
-# the next line restarts with tclsh \
-exec tclsh8.5 "$0" ${1+"$@"}
-
+#! /usr/local/bin/tclsh8.6
+## -*- tcl -*-
 # saslclient.tcl - Copyright (C) 2005 Pat Thoyts <patthoyts@users.sf.net>
 #
 # This is a SMTP SASL test client. It connects to a SMTP server and uses 
