@@ -1,7 +1,5 @@
-#! /bin/sh
-# the next line restarts with tclsh \
-exec tclsh8.5 "$0" ${1+"$@"}
-
+#! /usr/local/bin/tclsh8.6
+## -*- tcl -*-
 # MAIN PROGRAM
 #
 # Usage:
