@@ -1,6 +1,5 @@
-#!/bin/sh
-# -*- tcl -*- \
-exec tclsh8.5 "$0" ${1+"$@"}
+#! /usr/local/bin/tclsh8.6
+## -*- tcl -*-
 
 package require comm
 package require tie
