@@ -1,7 +1,5 @@
-#!/bin/sh
-# -*- tcl -*- \
-exec tclsh8.5 "$0" ${1+"$@"}
-
+#! /usr/local/bin/tclsh8.6
+## -*- tcl -*-
 # Array server ...
 
 package require comm
